﻿Socialist = LibStub("AceAddon-3.0"):NewAddon("Babel")

--------------------------------------------------------------------------------
--                            VARIABLE INITIALIZE                             --
--------------------------------------------------------------------------------

Socialist.initialized = false

local database_defaults = {
	realm = {
		whodb = {
			["next id"] = 1,
		},
		groupdb = {
			["next id"] = 1,
		},
		profdb = {
			["next id"] = 1,
		},
	},
	profile = {

	},
}

--------------------------------------------------------------------------------
--                                   SETUP                                    --
--------------------------------------------------------------------------------

function Socialist:OnIntialize()
	self.db = LibStub("AceDB-3.0"):New("SocialistDB", database_defaults,
	    true)
	self.whodb = self.db.realm.whodb
	self.groupdb = self.db.realm.groupdb
	self.profdb = self.db.realm.profdb

	self.initialized = true
end

function Socialist:OnEnable()

end

--------------------------------------------------------------------------------
--                                    API                                     --
--------------------------------------------------------------------------------


--[[ Miscellaneous Functions ]]--

function Socialist:ClearDB(database)
	if not self.initialized then return end
	if not database == "whodb" and
	   not database == "groupdb" and
	   not database == "profdb" then return end
	self.db.realm[database] = {}
end

--[[ WhoDB Functions ]]--

function Socialist:WhoDB_GetRecordByID(id)
	if not self.initialized then return end
	if not type(id) == "number" then return end
	if self.whodb["next id"] <= id then return end
	for k,v in ipairs(self.whodb) do
		if v.id == id then
			return v
		end
	end
end

function Socialist:WhoDB_Query(query,sort,order)
	if not self.initialized then return end
	local resultset = WhoDB_GatherResultSet(query)
	if sort then
		order = order or "DESCENDING"
		if sort == "name" then
			if order == "DESCENDING" then
				sort(resultset,nameSort_Desc)
			else
				sort(resultset,nameSort_Asc)
			end
		elseif sort == "guild" then
			if order == "DESCENDING" then
				sort(resultset,guildSort_Desc)
			else
				sort(resultset,guildSort_Asc)
			end
		elseif sort == "level" then
			if order == "DESCENDING" then
				sort(resultset,levelSort_Desc)
			else
				sort(resultset,levelSort_Asc)
			end
		elseif sort == "class" then
			if order == "DESCENDING" then
				sort(resultset,classSort_Desc)
			else
				sort(resultset,classSort_Asc)
			end
		elseif sort == "race" then
			if order == "DESCENDING" then
				sort(resultset,raceSort_Desc)
			else
				sort(resultset,raceSort_Asc)
			end
		elseif sort == "lastseen" then
			if order == "DESCENDING" then
				sort(resultset,lastseenSort_Desc)
			else
				sort(resultset,lastseenSort_Asc)
			end
		elseif sort == "zone" then
			if order == "DESCENDING" then
				sort(resultset,zoneSort_Desc)
			else
				sort(resultset,zoneSort_Asc)
			end
		end
	end
	return resultset
end

function Socialist:WhoDB_Modify(id,zone,info)
	if not self.initialized then return end
end

function Socialist:WhoDB_CreateNewPlayer(name,zone,info)
	if not self.initialized then return end

end

function Socialist:WhoDB_Remove(id)
	if not self.initialized then return end

end

--[[ GroupDB Functions ]]--
function Socialist:GroupDB_GetRecordByID(groupid)
	if not self.initialized then return end

end

function Socialist:GroupDB_Query(query)
	if not self.initialized then return end

end

function Socialist:GroupDB_Modify(groupid,info)
	if not self.initialized then return end

end

function Socialist:GroupDB_CreateNewGroup()
	if not self.initialized then return end

end

function Socialist:GroupDB_Remove(groupid)
	if not self.initialized then return end

end

function Socialist:GroupDB_AddPlayer(groupid,id)
	if not self.initialized then return end

end

function Socialist:GroupDB_AddRaidID(groupid,raidid)
	if not self.initialized then return end

end

function Socialist:GroupDB_AddInstance(groupid,instance)
	if not self.initialized then return end

end

--[[ ProfDB Functions ]]--
function Socialist:ProfDB_GetRecordByID(profid)
	if not self.initialized then return end

end

function Socialist:ProfDB_Query(query,sort,order)
	if not self.initialized then return end

end

function Socialist:ProfDB_Modify(profid,info)
	if not self.initialized then return end

end

function Socialist:ProfDB_Remove(profid)
	if not self.initialized then return end

end

function Socialist:ProfDB_CreateNewRecipe(name,info)
	if not self.initialized then return end

end

function Socialist:ProfDB_AddIngredient(profid,name,amount)
	if not self.initialized then return end

end

function Socialist:ProfDB_AddCrafter(profid,id)
	if not self.initialized then return end

end

--------------------------------------------------------------------------------
--                            Sorting Functions                               --
--------------------------------------------------------------------------------

local function nameSort_Desc(a,b)
	return a.name < b.name
end

local function nameSort_Asc(a,b)
	return a.name > b.name
end

local function guildSort_Desc(a,b)
	return a.guild < b.guild
end

local function guildSort_Asc(a,b)
	return a.guild > b.guild
end

local function levelSort_Desc(a,b)
	return a.level < b.level
end

local function levelSort_Asc(a,b)
	return a.level > b.level
end

local function classSort_Desc(a,b)
	return a.class < b.class
end

local function classSort_Asc(a,b)
	return a.class > b.class
end

local function raceSort_Desc(a,b)
	return a.race < b.race
end

local function raceSort_Asc(a,b)
	return a.race > b.race
end

local function lastseenSort_Desc(a,b)
	return a.lastseen < b.lastseen
end

local function lastseenSort_Asc(a,b)
	return a.lastseen > b.lastseen
end

local function zoneSort_Desc(a,b)
	return a.zone < b.zone
end

local function zoneSort_Asc(a,b)
	return a.zone > b.zone
end

--------------------------------------------------------------------------------
--                            Helper Functions                                --
--------------------------------------------------------------------------------

local function WhoDB_GatherResultSet(query)
	local result = {}
	for k,v in ipairs(Socialist.whodb) do
		local hit = true
		if hit and query.lastseen then
			hit = query.lastseen >= v.lastseen and hit or false
		end
		if hit and (query.levellow or query.levelhigh) then
			query.levelhigh = query.levelhigh or 80
			query.levellow = query.levellow or 0
			hit = query.levelhigh >= v.level and query.levellow <= v.level and
			    hit or false
		end
		if hit and query.class then
			hit = strfind(strupper(v.class),strupper(query.class)) and
			    hit or false
		end
		if hit and query.race then
			hit = strfind(strupper(v.race),strupper(query.race)) and
			    hit or false
		end
		if hit and query.zone then
			hit = strfind(strupper(v.zone),strupper(query.zone)) and
			    hit or false
		end
		if hit and query.guild then
			hit = strfind(strupper(v.guild),strupper(query.guild)) and
			    hit or strfind(strupper(v.normalguild),strupper(query.guild))
		end
		if hit and query.name then
			hit = strfind(strupper(v.name),strupper(query.name)) and
			    hit or strfind(strupper(v.normalname),strupper(query.name))
		end
		-- Pruning complete
		if hit then
			tinsert(result,v)
		end
	end
end



